/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "impl.h"
#include <stddef.h>

WOFS_IMPL int8_t wofs__dflt_dir_layout(WOFS_STATE_ARG wofs_addr_t addr,
	wofs_ino_t cur_ino, wofs_ino_t par_ino)
{
	for(int n_dots=1;n_dots<=2;++n_dots)
	{
		/* entry */
		{
			struct wofs_rawdirent de;

			de.d_ino     = (1==n_dots) ? cur_ino : par_ino;
			de.d_namelen = n_dots;
			de.d_alive   = 0xFF;

			if(0!=wroncefs_op_write(PASS_STATE &de, addr,
				sizeof(de)-sizeof(de.d_name2)))
			{
				return -WOFS_ERR_EIOW;
			}

			addr += sizeof(de)-sizeof(de.d_name2);
		}

		/* name */
		{
			char name[2];

			name[0] = '.';
			name[1] = '.';

			if(0!=wroncefs_op_write(PASS_STATE name, addr, n_dots))
			{
				return -WOFS_ERR_EIOW;
			}

			addr += 2;
		}
	}

	return 0;
}
