/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "impl.h"
#include <stddef.h>
#include <string.h>

WOFS_IMPL int8_t wofs__check_dir_RM(WOFS_STATE_ARG wofs_ino_t ino)
{
	{
		const int8_t ec = wofs__check_inode(PASS_STATE ino, WOFS_INOTYPE_DIR);

		if(0!=ec)
		{
			return ec;
		}
	}

	struct wofs_dir d;

	wofs__dir_open(PASS_STATE &d, ino, 0, NULL);

	for(;;)
	{
		const int8_t ec = wofs__dir_read(PASS_STATE &d);

		if((-WOFS_ERR_ENOMORE)==ec)
		{
			/* we're good */
			break;
		}
		else
		if(0!=ec)
		{
			return ec;
		}

		/* check "." */

		if(1==d.dir_ent.d_namelen && '.'==d.dir_ent.d_name[0])
		{
			continue;
		}

		/* check ".." */

		if(2==d.dir_ent.d_namelen && '.'==d.dir_ent.d_name[0] &&
			'.'==d.dir_ent.d_name[1])
		{
			continue;
		}

		/* it is something else */

		return -WOFS_ERR_ENOTEMPTY;
	}

	return 0;
}
