/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "impl.h"

#if WRONCEFS_SHORT_ERROR_STRINGS
#  define STR(_short_, _long_) _short_
#else /* WRONCEFS_SHORT_ERROR_STRINGS */
#  define STR(_short_, _long_) _long_
#endif /* WRONCEFS_SHORT_ERROR_STRINGS */

WOFS_IMPL const char* wofs_strerror(int8_t err)
{
	switch(err)
	{
	case 0:
		return STR("Succ", "Success");

	case WOFS_ERR_ENOSYS:
		return STR("ENOSYS", "Function not implemented");

	case WOFS_ERR_EBADSIG:
		return STR("EBADSIG", "Bad filesystem signature");

	case WOFS_ERR_EBADINFO:
		return STR("EBADINFO", "Bad or missing filesystem info");

	case WOFS_ERR_EINVAL:
		return STR("EINVAL", "Invalid argument");

	case WOFS_ERR_EIOR:
		return STR("EIOR", "I/O error during read");

	case WOFS_ERR_EIOW:
		return STR("EIOW", "I/O error during write");

	case WOFS_ERR_EIOE:
		return STR("EIOE", "I/O error during erase");

	case WOFS_ERR_ENOENT:
		return STR("ENOENT", "No such file or directory");

	case WOFS_ERR_EEXIST:
		return STR("EEXIST", "File exists");

	case WOFS_ERR_ENOTDIR:
		return STR("ENOTDIR", "Not a directory");

	case WOFS_ERR_EFBIG:
		return STR("EFBIG", "File too large");

	case WOFS_ERR_EISDIR:
		return STR("EISDIR", "Is a directory");

	case WOFS_ERR_ENOTEMPTY:
		return STR("ENOTEMPTY:", "Directory not empty");

	case WOFS_ERR_ENOTRECOVERABLE:
		return STR("ENOTRECOV", "State not recoverable");

	case WOFS_ERR_ENOSPC:
		return STR("ENOSPC", "No space left on device");

	case WOFS_ERR_ENAMETOOLONG:
		return STR("ENAMTOOLON", "Filename too long");

	case WOFS_ERR_EBADINODE:
		return STR("EBADINODE", "Damaged i-node structure");

	case WOFS_ERR_ENOMORE:
		return STR("ENOMORE", "No more entries in the directory");

	case WOFS_ERR_EOVERFLOW:
		return STR("EOVRFLOW", "Value too large to be stored in data type");

	case WOFS_ERR_ENODEV:
		return STR("ENODEV", "No such device");

	case WOFS_ERR_EUCLEAN:
		return STR("EUCLEAN", "Structure needs cleaning");

	default:
		break;
	}

	return STR("???", "Unknown error");
}
